module namespace dx = "http://schemas.epocrates.com/schemas/xquery/dx";

import module namespace global = "http://schemas.epocrates.com/schemas/xquery/dxglobal"
   at "global.xquery" ;


declare variable $dx:mono := /monographs/element()[1];
declare variable $dx:dx_id := xs:integer($dx:mono/@dx_id);
declare variable $dx:pages := $global:pages;
declare variable $dx:creator := $global:creator ;
declare variable $dx:popheight := $global:popheight ;

declare variable $dx:mono_name :=$dx:mono/title/string();

declare variable $dx:tx_view_start := 20 ;

(: simple error :)
declare function dx:error( $e as xs:string , $d )
{
	error( xs:QName( "dx:error"), concat($e,":",$d) ) 
};

(: equality test for names (pt groups and parent groups)
  use normalized-space and case insensitive compare
  :)
declare function dx:name-equals( $s1 as xs:string ?, $s2 as xs:string ) as xs:boolean
{
	fn:upper-case( fn:normalize-space( $s1 ) ) eq 
	fn:upper-case( fn:normalize-space( $s2 ) )  

};


(: format link tags :)
declare function dx:link( $e as element(link) ) as node()
{
	$e/text()

};

(: format organism tag :)
declare function dx:organism( $e as element(organism) ) as node()
{
	$e/text()

};

(: format images tag :)
declare function dx:images( $e as element(images) )
{
	()

};
(: format footnote tag :)
declare function dx:foot( $e as element(foot) )
{
	()

};

(: Normalize adjacent text nodes to one text node :)

declare function dx:normalize-text( $nodes as node()* ) as node()*
{
  if( count( $nodes ) lt 2 )  then 
     $nodes
  else
   if( $nodes[1] instance of text() and $nodes[2] instance of text() ) then
	     dx:normalize-text( (text { fn:concat( $nodes[1] , $nodes[2] ) } , fn:subsequence($nodes , 3 ) ))
   else
        ( $nodes[1] , dx:normalize-text( subsequence( $nodes , 2 ) ) )

};



(: true if the all the nodes are a text node thats entirely blank :)
declare function dx:is-blank-text( $nodes as node()* ) as xs:boolean
{
empty(
  for $n in $nodes
  return 
     if( $n instance of text() and string-length(normalize-space($n)) eq 0 ) then 
	   () 
	 else 
	    $n
)
};


(: 
 	format list tag 
	<M>List Item</M>
	<M>List Item</M>
	<BR/>

	Note: if a list element is composed of entirely blank text nodes then skip it

:)
declare function dx:list( $e as element(list) )
{
	(
	for $item in $e/item
	let $p := dx:complex-nopara( $item/* )
	return 
	if( dx:is-blank-text( $p ) ) then 
	    () 
	else <M>{$p}</M>,
	<BR/>
	)

};



(: format %text_markup tags :)
declare function dx:text( $es  ) as node()*
{ 
	for $e in $es
	return
	typeswitch ($e)
	case	$t as text() return $t
	case	$l as element(link) return dx:link( $l )
	case	$o as element(organism) return dx:organism( $o )
	case	$f as element(foot) return dx:foot($f)
	default	
			return dx:text( $e/node() )
		
	
};



(: format %complex_text tags :)
declare function dx:complex( $es  ) as node()*
{ 
	for $e in $es
	return
	typeswitch ($e)
	case	$l as element(list) return dx:list( $l )
	case	$p as element(para) return dx:para($p)
	default	
			return dx:error( "unknown element in complex" , fn:node-name($e))
		
	
};



(: format %complex_text tags :)
declare function dx:complex-nopara( $es  ) as node()*
{ 
	for $e in $es
	return
	typeswitch ($e)
	case	$l as element(list) return dx:list( $l )
	case	$p as element(para) return dx:para_content( $p/node() )
	default	
			return dx:error( "unknown element in complex" , fn:node-name($e))
		
	
};



(: concatenate a name node's text and direct children as text :)
declare function dx:name( $n as element() ) as xs:string
{
	normalize-space(string($n))
};


(: format para content  :)
declare function dx:para_content( $p as node()* ) as node()*
{ 
	for $e in $p
	return
	typeswitch ($e)
	case	$t as text() return $t
	case	$l as element(link) return dx:link( $l )
	case	$f as element(foot) return dx:foot($f)
	case	$o as element(organism) return dx:organism( $o )
	case	$i as element(images) 	return dx:images( $i )
	default	
		return dx:error( "unknown element in para" , fn:node-name($e) )
	
};



(: format para tag :)
declare function dx:para( $p as element(para) ) as node()*
{ 
	(<P>{dx:para_content($p/node())}
	</P>,<BR/>)
	
};

(: convert <para> elements to <P> elements interspaced with <BR/> :)
declare function dx:para_text( $es as element(para)* ) as node()*
{ 
	for $p in $es
	return 	dx:para( $p ) 
};


declare function dx:section( $s as element(section) ) as node()*
{ 
	let $head := $s/section_header,
	    $text := $s/section_text
	return
(
	if( exists( $head ) ) then 
		<H1>{dx:text( $head/node() )}</H1>
	else () , 
	if( exists( $text ) ) then 
		dx:complex( $text/* ) 
	else ()
)


};

declare function dx:page_tree( $page as xs:string ) as element(page)+ 
{
	
	let $dummy := fn:trace( $page , 'page_tree' ),
		$p := dx:page($page),
	    $parent := $p/@parent
	return 
		if( empty($parent) ) then $p 
		else
			( dx:page_tree( $parent ) , <page>{$p/@*}</page> )

};

declare function dx:page_root( $page as xs:string ) as xs:string
{
	let $p := dx:page($page)
	return
	if( exists( $p/@parent ) ) then dx:page_root( $p/@parent )
	else	$page

};



(: Output a optional section jump line :)
declare function dx:optjump( $node , $name , $target )
{
	if( exists( $node ) ) then
		<L><A jump="{$target}">{$name}</A></L>
	else ()
};

(: create a link to a page within this monograph :)
declare function dx:pagelinkfull( $page as xs:string ) as element(A)
{
	let $title := exactly-one(dx:page($page)/@title/string())
	return
	<A page="{$dx:dx_id}-{$page}">{$title}</A>
};
(: return the offset from the beginning of a sequence 0 based of a node or -1 :)
declare function dx:offset( $node as node() ? ) as xs:integer
{
	if( exists( $node ) ) 
	then 	
		count($node/preceding-sibling::*) 
	else -1 
};


(: create a link to a page within this monograph :)
declare function dx:pagetarget( $pagename as xs:string ) as xs:string
{
	let $page := dx:page($pagename),
	    $view := $page/@view/string()
	return concat(".",$view)
};

declare function dx:page( $pagename as xs:string ) as element( page )
{
	$dx:pages/pages/page[@name=$pagename]
};


(: create a link to a page within this monograph :)
declare function dx:pagelink( $pagename as xs:string ) as element(A)
{
	let $page 	:= dx:page($pagename),
	    $title := $page/@title/string(),
	    $target := dx:pagetarget( $pagename )
	return
	<A page="{$target}">{$title}</A>
};

declare function dx:section_exists( $page as element(page) ) as xs:boolean 
{
	let $exists := $page/@exists/string()
	return
	if( string-length( $exists ) = 0 ) then fn:true()
	else
		exists( $dx:mono/saxon:evaluate($exists) )

};


declare function dx:xtext( $name as xs:string , $subtitle as xs:string, $page as element(PAGE)  ) as element( XTEXT )
{
<XTEXT bheight="10" lspacing="3" lr_hpad="8" tb_vpad="8" vspacing="0">

<PAGE bheight="10" lspacing="3" lr_hpad="8" tb_vpad="8" vspacing="0" title="{$dx:mono_name}" subtitle="{$subtitle}" >
{ $page/node() }
</PAGE>
</XTEXT>
};


declare function dx:xlist( $name as xs:string , $subtitle as xs:string, $list as element(XLIST) ) as element( XTEXT )
{
<XTEXT bheight="10" lspacing="3" lr_hpad="8" tb_vpad="8" vspacing="0">

<XLIST title="{$dx:mono_name}" subtitle="{$subtitle}" >
{ $list/node() }
</XLIST>
</XTEXT>
};


declare function dx:linkdx( $dx_id as xs:string? ) as attribute()*
{
	if( exists( $dx_id ) ) then 
    attribute { "href" } { concat($dx_id , ".html") }
	else ()

};
(: Stylus Studio meta-information - (c) 2004-2009. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\..\data\1.xml" outputurl="" processortype="saxon" tcpport="0" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
:)