import module namespace global = "http://schemas.epocrates.com/schemas/xquery/dxglobal"
   at "global.xquery" ;

declare variable $monos external;
declare variable $creator := $global:creator ;
declare variable $category_doc := doc("../../../data/categories.xml");
declare variable $sorted_categories := local:sorted_categories();
declare variable $sorted_monos := local:sort_mono(/);


declare function local:string( $s as xs:string ) as element()+
{
	
	<WORD value="{string-length( $s )}"/>,
	<STRING pad="0" value="{$s}"/>

};

declare function local:is_unique_syn( $syns , $s )
{
	if( $s/@synonym eq '' ) then fn:false()
	else
		fn:count( $syns/synonyms/topic[@synonym eq $s/@synonym] ) eq 1
};

declare function local:page( $id as xs:integer , $title as xs:string? ) as element(page)?
{
	if( empty($title) ) then 
		() (: <page foo="bar"/> :)
	else
	<page id="{$id}" title="{$title}"/>

};


declare function local:combine_syns( $syns ) as element(page)*
{
	for $page in $monos//page
	let $syn := $syns/synonyms/topic[@topic_id=$page/@id],  
	    $t :=  $global:topics//topic[@topic_id=$page/@id ],
		$pda_name :=  if( exists( $t/@pda_name ) )  
			then $t/@pda_name 	  
			else $t/@name

	return
(
	if( exists( $syn ) ) then 
	(

	  for $s in $syn 
	  where ( ($s/@synonym ne $pda_name) and 
	          local:is_unique_syn($syns , $s ) )
	  return 
	     local:page($page/@id,$s/@synonym)

	) else () 
	,
	 local:page($page/@id,$pda_name),
	    
	(: if pda_synonym then also use topic name as syn :)
	if( $t/@pda_synonym_flag eq "1" ) then 
		local:page($page/@id,$t/@name)
	else () 
	)
};
	


declare function local:sorted_categories() 
{
	for $c in $category_doc/categories/category
	order by fn:upper-case( $c/@name )
	return $c 
};

declare function local:topics_in_category( $c as element(category) )
{
	(: magic NxM linking :)
	let $topics := $sorted_monos[@id = $c/topic/@id ]
	for $t in $topics
	order by xs:integer($t/@pos )
	return $t
};


declare function local:sort_mono($topics) as element(page)*
{
    let $all := 
		for $m in local:combine_syns( $topics )
		order by fn:upper-case($m/@title)
		return 
			$m
	for $m at $pos in $all
	return <page pos="{$pos - 1}">{$m/@*}</page>

};



<XMLPDB>
<PDB name="TopicsList-{$creator}" creator="{$creator}" type="tlst">
{
	for $m at $pos in $sorted_monos
	return
	<RECORD>
		<WORD value="{$m/@pos }"/>
	{ 
		local:string( $m/@id ) ,
		  local:string( $m/@title ) 
	  }
	</RECORD>

}
</PDB>
<PDB name="TopicsSubKey-{$creator}" creator="{$creator}" type="tkey">
	<RECORD>
	  <WORD value="0"/>
	  <WORD value="{count($sorted_monos)}"/>
	{
 		for $i in 0 to count($sorted_monos) - 1
		return
		    <WORD value="{$i}"/>

	}
	</RECORD>
{
	(: RECORD 1 ... nspecs - List of all specimen sub-keys :) 
	for $c at $pos in $sorted_categories
	return
		<RECORD comment="{$c/@name}">
			<WORD value="{$pos}"/>
			
			{ 
				let $ts := local:topics_in_category( $c )
				return (
					<WORD value="{count($ts)}" /> ,

					for $t in $ts
					return
						<WORD value="{$t/@pos}" comment="{$t/@title}"/> 
				)
				
			}
		</RECORD>
}


</PDB>


<PDB name="CategoryList-{$creator}" creator="{$creator}" type="clst">
{
  for $c at $pos in $sorted_categories
  return 
  <RECORD>
     <WORD value="{$pos - 1}"/>
	 {local:string($c/@name) }
	 <WORD value="{$pos}"/>
  </RECORD>
}
</PDB>

</XMLPDB>(: Stylus Studio meta-information - (c) 2004-2008. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\data\synonyms.xml" outputurl="" processortype="saxon" tcpport="0" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<parameterValue name="monos" value="doc('..\..\other\monos.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="true"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
:)