import module namespace dx = "http://schemas.epocrates.com/schemas/xquery/dx"
   at "common.xquery" ;

declare variable $options := $dx:mono//treatment/tx_options;
declare variable $presumptive 	:= $options/tx_option[timeframe='presumptive'];
declare variable $ongoing 		:= $options/tx_option[timeframe='ongoing'];
declare variable $acute 		:= $options/tx_option[timeframe='acute'];
declare variable $tx_id  external ;

(: declare variable $drug_map := doc("../../../data/drugs.xml"); :)

declare variable $drug_map  external ;



(: Return name possibly as a link if it links to rx exactly :)
declare function local:maylinkrx( $name as xs:string ) 
{
    let $rxname := upper-case(fn:normalize-space($name)),
	    $drug := $drug_map/drugs/drug[upper-case(@name) eq $rxname]
	return
	if( exists( $drug ) ) then
		<A app="rx" args="-d {$drug/@id - 1}">{$name}</A>
	else
		$name 

};




declare function local:regimen( $reg as element(regimen))
{
	for $c  in $reg/components/component
  	return (
		local:maylinkrx($c/name),
		if( exists( $c/details ) ) then concat(" : " ,	$c/details/string() ) else () ,
		if( exists( $c/@modifier) ) then 
		   if( $c/@modifier = ("AND","OR") ) then 
		     (<BR/>,<L><B>-- {$c/@modifier/string()} --</B></L>)
		   else
		     ( text{" "} , <B>{$c/@modifier/string()}</B> , <BR/>) 
		else ()

	)
};




declare function local:tx_type( $t as element(tx_option) ) as xs:string
{
   $t/tx_type/string()
};

declare function local:tx_line( $t as element( tx_option ) ) as xs:string?
{
	let $txl := $t/tx_line/string()
	return
	if( $txl eq "1" ) then "1st" 
	else
	if( $txl eq "2" ) then "2nd" 
	else
	if( $txl eq "3" ) then "3rd" 
	else
	if( $txl eq "4" ) then "4th" 
	else
	if( $txl eq "5" ) then "5th" 
	else
	if( $txl eq "6" ) then "6th" 
	else
	if( $txl eq "7" ) then "7th" 
	else
	if( $txl eq "8" ) then "8th" 
	else
	if( $txl eq "P" ) then "plus" 
	else
	if( $txl eq "A" ) then "adj."
	else
		$txl 


};



(: Return all options that have non-blank tx_lines :)

declare function local:tx_lines( $txs as element( tx_option)* ) as element(tx_option)*
{

	$txs[ exists( tx_line ) ] 

};



(: return normalized pt_group name because the XML data is bad and has leading/trailing blanks :)
declare function local:pt_group_name( $tx as element( tx_option ) ) as xs:string
{
	normalize-space( $tx/pt_group/string() )
};

declare function local:pt_parent_name( $tx as element( tx_option ) ) as xs:string
{
	normalize-space( $tx/parent_pt_group/string() )
};
(: Collect tier options 
  Note: As per email 11/26/2008 - 4th and + tiers will be combined with 3rd tier
  :)

declare function local:tier( $t as element(tx_option) , $tier as xs:integer  ) as element(regimen)*
{
	if( $tier eq 3 ) then 
		$t/regimens/regimen[ xs:integer(normalize-space(@tier) ) gt 2 ]
	else 
		$t/regimens/regimen[ xs:integer(normalize-space( @tier) ) = $tier ] 
};


declare function local:tx_detail( $t as element(tx_option) ) as element(POPUP)?
{
(: Only create a popup if it has data :)
if( exists( $t/comments union $t/regimens ) ) then 

<POPUP tag="p_{dx:offset($t)}" title="{$dx:mono_name}" subtitle="Treatment Detail: {$t/tx_type/string()}" maxheight="{$dx:popheight}">
{ dx:para_text( $t/comments/* ) }
{ 
	let $tier1 := local:tier( $t , 1 ),
	    $tier2 := local:tier( $t , 2 ),
	    $tier3 := local:tier( $t, 3 )

	return 
(
	if( exists( $tier1 ) ) then (
		<H1>Primary Options</H1>,
		for $reg in $tier1 
		return (<M>{local:regimen( $reg )}</M>,<BR/>)
	) else ()
	,
	if( exists( $tier2 ) ) then (
		<H1>Secondary Options</H1>,
		for $reg in $tier2 
		return (<M>{local:regimen( $reg )}</M>,<BR/>)
	) else ()
	,
	if( exists( $tier3 ) ) then (
		<H1>Tertiary Options</H1>,
		for $reg in $tier3 
		return (<M>{local:regimen( $reg )}</M>,<BR/>)
	) else ()
(:
	,
	if( exists( $tier4 ) ) then (
		<H1>Quaternary Options</H1>,
		for $reg in $tier4 
		return (<M>{local:regimen( $reg )}</M>,<BR/>)
	) else ()
:)


)
}
</POPUP>

else ()

};



declare function local:tx_content( 
	$tx as element( tx_option )+ , 
	$parent_lines as element(tx_option)*  )
{
	let $my_lines := local:tx_lines($tx)
	return

	if( exists( $my_lines ) ) then  (
		
	  	if( exists( $parent_lines ) ) then 
     		for $t in $parent_lines 
			return
			<CELL>
				{
			    (: Only add a popup link if this contains comments or regimins :)
				if( exists( $t/comments union $t/regimens ) ) then 
					attribute { "popup" } { concat("p_" , dx:offset($t) ) }
				else () 
				}


			   <TEXT width="25" size="small" xpos="0" text="{local:tx_line($t)}"/>
			   <TEXT width="75" xpos="25" text="{local:tx_type($t)}"/>
			</CELL>
		else ()
		,
		for $t in $my_lines
		return
			<CELL>
			{
			    (: Only add a popup link if this contains comments or regimins :)
				if( exists( $t/comments union $t/regimens ) ) then 
					attribute { "popup" } { concat("p_" , dx:offset($t) ) }
				else () 
			}


			 	<TEXT width="25" size="small" xpos="0" text="{local:tx_line($t)}"/>
			   <TEXT width="75" xpos="25" text="{local:tx_type($t)}"/>	
			</CELL>
	)
	
	else ()
	(: dx:text( $tx[1]/tx_type ) :)
		

};

(: Format the entry for a single child group :)
declare function local:tx_sect_child(  
	$tx_child as element(tx_option )+ , 
	$parent_lines as element(tx_option)*  )
{
	
	local:tx_content( $tx_child , $parent_lines  )

};




(: Format the link for a single parent group , may be multiple tx_options :)

declare function local:tx_sect_parent(  
	$tx_parent as element(tx_option )* , 
	$tx_children as element( tx_option )* 
)
{
		let $parent_lines := local:tx_lines( $tx_parent ),
			$parent_content := local:tx_content( $tx_parent , ()  )
			
		return  
		<GROUP heading="">
		  <CELL >
			<TEXT fgcolor="header" bold="true"  text="{local:pt_group_name($tx_children[1])}"/>
	     </CELL>
		 {
		 	let $child_groups := distinct-values( 
				for $c in $tx_children return fn:upper-case(local:pt_group_name($c))
				 )
			for $g in $child_groups 
			let $child := $tx_children[ dx:name-equals(pt_group,$g) ]
			return
			local:tx_sect_child( $child , $parent_lines  )

		}
		</GROUP>
		

};

(:
 Create popup (detail) elements only for those options which will have 
 a link created in the XLIST
 MUST follow exactly the same logic as is followed to create links
 :)


declare function local:all_details(
    $tx_parent as element(tx_option )* , 
	$tx_children as element( tx_option )* 
) as element(POPUP)*
{
(

	let $parent_lines := local:tx_lines( $tx_parent )
	for $t in $parent_lines 
	return local:tx_detail($t) 
	,

	let $child_groups := distinct-values( 
		for $c in $tx_children return fn:upper-case(local:pt_group_name($c)) )
	for $g in $child_groups 
	  let $child := $tx_children[ dx:name-equals(pt_group,$g) ],
	      $child_lines := local:tx_lines( $child )
	   return
	 	 	for $t in $child_lines
				return local:tx_detail($t) 

)



};



dx:xlist("tx_options_details","Treatment",
<XLIST>

{
   let 
   $child := $options/tx_option[ xs:integer($tx_id) + 1 ] , 
   $child_name := local:pt_group_name( $child ),
   $pg := local:pt_parent_name($child),
   $children := $options/tx_option[ dx:name-equals(pt_group,$child_name) and 
   									dx:name-equals(parent_pt_group,$pg) ],
   $parent := $options/tx_option[ dx:name-equals(pt_group,$pg)] 
   return (
       local:all_details( $parent , $children ),
       local:tx_sect_parent(  $parent , $children )
   )

}
</XLIST>
)
(: Stylus Studio meta-information - (c) 2004-2009. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="no" name="150" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\150.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="1"/>
			<parameterValue name="drug_map" value="doc('..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="120" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\XMLSource\120.xml" outputurl="" processortype="saxon" tcpport="0" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
		</scenario>
		<scenario default="no" name="101" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\XMLSource\101.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
		</scenario>
		<scenario default="no" name="616" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\616.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="1"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="61" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\XMLSource\61.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="549" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\XMLSource\549.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="1" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\1.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="0"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="44" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\44.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="0"/>
			<parameterValue name="drug_map" value="doc('..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="350" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\350.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="6"/>
			<parameterValue name="drug_map" value="doc('..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="728" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\728.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="2"/>
			<parameterValue name="drug_map" value="doc('..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="561" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\xml\561.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="3"/>
			<parameterValue name="drug_map" value="doc('..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" useresolver="no" url="..\..\..\..\..\..\feature\full_iPhoneEssv2\DX\xml\964.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0"
		          profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user=""
		          password="" validateoutput="yes" validator="internal" customvalidator="">
			<parameterValue name="tx_id" value="0"/>
			<parameterValue name="drug_map" value="doc('..\..\..\data\drugs.xml')"/>
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="bStrip" value="true"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
:)