import module namespace dx = "http://schemas.epocrates.com/schemas/xquery/dx"
   at "common.xquery" ;

declare variable $options := $dx:mono//treatment/tx_options;
declare variable $presumptive 	:= $options/tx_option[timeframe='presumptive'];
declare variable $ongoing 		:= $options/tx_option[timeframe='ongoing'];
declare variable $acute 		:= $options/tx_option[timeframe='acute'];




declare function local:regimen( $reg as element(regimen))
{
	for $c  in $reg/components/component
  	return (
		dx:name($c/name),
		if( exists( $c/details ) ) then concat(" : " ,	$c/details/string() ) else () ,
		if( exists( $c/@modifier) ) then concat(" " , $c/@modifier , " ") else ()

	)
};

(: return normalized pt_group name because the XML data is bad and has leading/trailing blanks :)
declare function local:pt_group_name( $tx as element( tx_option ) ) as xs:string
{
	normalize-space( $tx/pt_group/string() )
};

declare function local:pt_parent_name( $tx as element( tx_option ) ) as xs:string
{
	normalize-space( $tx/parent_pt_group/string() )
};




declare function local:txpage( $t as element(tx_option) ) as xs:string
{


  concat("." , dx:offset($t) + $dx:tx_view_start )

};



(: Format the link for a single parent group , may be multiple tx_options :)

declare function local:tx_sect_parent(  
	$tx_parent as element(tx_option )+ , 
	$tx_children as element( tx_option )* , 
	$pgid 
) as element(CELL)*
{
(
    (: top level group cell - linkable if any tx_lines :)
	<CELL  >
	  { if( exists( $tx_parent[1]/tx_line ) ) then 
	      attribute { "page" } { local:txpage( $tx_parent[1] ) }
	  else ()  }
	  <TEXT fgcolor="header" bold="true"  text="{local:pt_group_name($tx_parent[1])}"/>
	</CELL>,
	let $child_groups := distinct-values( for $c in $tx_children return fn:upper-case(local:pt_group_name($c)) )
	for $g at $pos in $child_groups 
	let $child := $tx_children[ dx:name-equals(pt_group,$g) ]
	return
		<CELL page="{local:txpage($child[1])}">
			<TEXT xpos="5" width="95" text="{local:pt_group_name($child[1])}" />
		</CELL>

)
};

(: Format the content for a given section ("Ongoing" ... ) :)

declare function local:tx_sect( $tx_option as element( tx_option)* , 
		$title as xs:string , 
		$tag as xs:string ) as element(GROUP)*
{
	if( exists( $tx_option ) ) then (
		<GROUP heading="{$title}">
		{
			(: Get distinct top level groups only :)
			let $groups := distinct-values( 
				for $c in $tx_option[empty(parent_pt_group)] 
				return fn:upper-case(local:pt_group_name($c)) )

			for $g at $pos in $groups 
			let $parent := $tx_option[ dx:name-equals(pt_group,$g) and empty(parent_pt_group) ],
		    	$children := $tx_option[ dx:name-equals(parent_pt_group,$g) ]
			return 
				local:tx_sect_parent(  $parent , $children , concat( $tag , "_" , $pos  ) )
		}
		</GROUP>		
	) else ()
};



dx:xlist("tx_options", "Treatment Options",
<XLIST>
{
	local:tx_sect( $presumptive , "Presumptive" , "presumptive"  ),
	local:tx_sect( $acute , "Acute" , "acute" ),
	local:tx_sect( $ongoing , "Ongoing" , "ongoing" )
}

</XLIST>
)
(: Stylus Studio meta-information - (c) 2004-2008. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="no" name="150" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\..\xml\150.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="120" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\XMLSource\120.xml" outputurl="" processortype="saxon" tcpport="0" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
		</scenario>
		<scenario default="no" name="101" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\XMLSource\101.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
		</scenario>
		<scenario default="no" name="616" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\XMLSource\616.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="61" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\XMLSource\61.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="549" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\..\xml\549.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="3" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\..\xml\3.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="yes"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<validatorSchema value="..\..\..\..\..\..\..\perforce\eng\Schemas\main\xtext_1_2.xsd"/>
		</scenario>
		<scenario default="no" name="44" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\..\xml\44.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
		</scenario>
		<scenario default="yes" name="728" userelativepaths="yes" externalpreview="no" useresolver="yes" url="..\..\xml\728.xml" outputurl="" processortype="saxon" tcpport="60291088" profilemode="0" profiledepth="" profilelength="" urlprofilexml=""
		          commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" host="" port="0" user="" password="" validateoutput="no"
		          validator="internal" customvalidator="">
			<advancedProperties name="bSchemaAware" value="false"/>
			<advancedProperties name="bXml11" value="false"/>
			<advancedProperties name="iValidation" value="0"/>
			<advancedProperties name="bExtensions" value="true"/>
			<advancedProperties name="iWhitespace" value="0"/>
			<advancedProperties name="bTinyTree" value="false"/>
			<advancedProperties name="bUseDTD" value="false"/>
			<advancedProperties name="bWarnings" value="true"/>
		</scenario>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
:)